@@ -84,6 +84,7 @@
 
 #if defined(LINUX)
 #define PROCFS_TEMPLATE "/proc/%d/stat"
+#define PROCFS_CMDLINE_TEMPLATE "/proc/%d/cmdline"
 #endif /* defined(LINUX) */
 
 #if defined(FREEBSD)
@@ -462,6 +463,7 @@ int process_parse_procfs(struct process *process) {
 #if defined(LINUX)
     char *r,*q;
     char deparenthesised_name[WMTOP_BUFLENGTH];
+		int endl;
 #endif /* defined(LINUX) */
 #if defined(FREEBSD)
     int us,um,ks,km;
@@ -513,10 +515,51 @@ int process_parse_procfs(struct process *process) {
      * Remove parentheses from the process name stored in /proc/ under Linux...
      */
     r = procname+1;
-    q = deparenthesised_name;
-    while (*r && *r!=')')
-	*q++ = *r++;
-    *q = 0;
+    /* remove any "kdeinit: " */
+    if (r == strstr(r, "kdeinit"))
+     {
+      sprintf(filename,PROCFS_CMDLINE_TEMPLATE,process->pid);
+
+      /*
+       * Permissions of /proc filesystem are permissions of process too
+       */
+      if (user>=0) {
+        stat(filename,&sbuf);
+        if (sbuf.st_uid!=user)
+            return 1;
+      }
+
+      ps = open(filename,O_RDONLY);
+      if (ps<0)
+        /*
+         * The process must have finished in the last few jiffies!
+         */
+        return 1;
+
+        endl = read(ps,line,sizeof(line));
+       close(ps);
+
+      /* null terminate the input */
+      line[endl]=0;
+      /* account for "kdeinit: " */
+      if ((char*)line == strstr(line, "kdeinit: "))
+        r = ((char*)line)+9; 
+      else
+        r = (char*)line;
+
+      q = deparenthesised_name;
+      /* stop at space */
+      while (*r && *r!=' ')
+        *q++ = *r++;
+       *q = 0;
+    }
+    else
+    {
+      q = deparenthesised_name;
+      while (*r && *r!=')')
+        *q++ = *r++;
+       *q = 0;
+    }
 
     if (process->name)
 	wmtop_free(process->name);
